/* (C) Zx-81 2012 - 2013

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.  Redistributions in binary
form must reproduce the above copyright notice, this list of conditions and the
following disclaimer in the documentation and/or other materials provided with
the distribution.

THIS SOFTWARE IS PROVIDED BY THE 63IN1 PROJECT ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <coleco.h>
#include <getput1.h>

#include "types.h"
#include "utils.h"
#include "main.h"
#include "loader.h"
#include "game.h"

void nmi(void) {}

/* Couleurs

0 Invisible - - -
1 Black 0.00 0.47 0.47
2 Medium Green 0.53 0.07 0.20
3 Light Green 0.67 0.17 0.27
4 Dark blue 0.40 0.40 1.00
5 Light blue 0.53 0.43 0.93
6 Dark Red (brown) 0.47 0.83 0.30
7 Cyan 0.73 0.00 0.70
8 Medium Red 0.53 0.93 0.27
9 Light Red (Pink/orange) 0.67 0.93 0.27
10 (A) Dark Yellow (Yellow) 0.73 0.57 0.07
11 (B) Light Yellow (Yellow + Light Grey) 0.80 0.57 0.17
12 (C) Dark Green 0.47 0.13 0.23
13 (D) Magenta 0.53 0.73 0.67
14 (E) Grey (Light Grey) 0.80 0.47 0.47
15 (F) White

*/

# ifndef COLOR
# define COLOR 1
# endif

void 
init_screen()
{
  disable_nmi();
  // Affichage 32 colonnes x24 lignes  [0:31] [0:23]
	screen_mode_1_text();
	upload_default_ascii(NORMAL);
  // vert vif
  // paper(2);
  // vert fonce
  // paper(12);
  paper(COLOR);
  enable_nmi();

  cls();
}

void
init_sound()
{
  play_sound ( 1 );
  play_sound ( 2 );
}

void
display_banner_dx( uchar dx, uchar page )
{
  char  full_text[128];
  char  banner[33];

  uchar len;
  uchar i;

  print_at( 0, 1, " 63 GAMES IN 1 CARTRIDGE    " );
  strcpy(banner, "(x/2)");
  banner[1] = '0' + page;
  print_at( 27, 1, banner );

  strcpy( full_text, "(C) Buzz & ZX-81    2013/01 v1.0  *** PAD TO MOVE & FIRE TO START GAME *** " );
  len = strlen( full_text );
  dx = (dx >> 1) % len;
  i = 0;
  while (i < 32) {
    if (dx >= (len - 1)) dx = 0;
    banner[i++] = full_text[dx++];
  }
  banner[i] = 0;
  center_string(23 , banner );
}

void
display_banner()
{
  center_string(1 , "*** 63 GAMES IN 1 CARTRIDGE  ***" );
  center_string(23, "(C) Buzz & ZX-81    2013/01 v1.0" );
}

void 
display_game_list( uchar game_sel )
{
  uchar x;
  uchar y_min = 4;
  uchar y;
  uchar game_id;
  uchar game_offset;
  char  game_name[32];

  x = 1;
  y = y_min;

  game_offset = 0;
  if (game_sel >= GAME_MAX_PAGE) {
    game_offset = GAME_MAX_PAGE;
    game_sel -= GAME_MAX_PAGE;
  }

  disable_nmi();
  // Display game list
  for (game_id = 0; game_id < GAME_MAX_PAGE; game_id++) {
    game_name[0] = ' ';
    if (game_sel == game_id) game_name[0] = '>';
    get_game_name( game_id + game_offset, game_name + 1);
    print_at( x, y, game_name );
    y++;
    if ((game_id+1) == GAME_MAX_LINE) { y = y_min; x = 17; }
  }
  enable_nmi();
  delay(1);
}

uchar
load_game( uchar game_sel )
{
  char   game_name[32];  
  ushort bank_addr;
  // char*  p;

	disable_nmi();
  cls();
  display_banner();

  center_string( 11, "Loading  " );
  get_game_name( game_sel, game_name);
  center_string( 12, game_name);
  center_string( 13, "Please Wait" );
  enable_nmi();
	delay(50);

  disable_nmi();
  // loader is 63
  bank_addr =  0xffc0 + game_sel;

  game_loader_exec( bank_addr );

  return 0;
}

uchar
get_game_select( uchar* p_game_sel )
{
  uchar  joy_evt  = 0;
  uchar  game_sel = *p_game_sel;
  uchar  game_offset = 0;

  joy_evt = joypad_1;
  if ((joy_evt & COL_JOY_UP)) {
    game_sel = (game_sel + GAME_LIST_SIZE - 1) % GAME_LIST_SIZE;
  } else 
  if ((joy_evt & COL_JOY_DOWN)) {
    game_sel = (game_sel + 1) %  GAME_LIST_SIZE;
  } else 
  if ((joy_evt & COL_JOY_LEFT)) {

    if (game_sel >= GAME_MAX_PAGE) {
      game_offset = GAME_MAX_PAGE;
      game_sel -= GAME_MAX_PAGE;
    }
    game_sel = (game_sel + GAME_MAX_PAGE - GAME_MAX_LINE) % GAME_MAX_PAGE;
    game_sel = (game_sel + game_offset                  ) % GAME_LIST_SIZE;

  } else 
  if ((joy_evt & COL_JOY_RIGHT)) {

    if (game_sel >= GAME_MAX_PAGE) {
      game_offset = GAME_MAX_PAGE;
      game_sel -= GAME_MAX_PAGE;
    }
    game_sel = (game_sel + GAME_MAX_LINE  ) % GAME_MAX_PAGE;
    game_sel = (game_sel + game_offset    ) % GAME_LIST_SIZE;

  } else 
  if ((joy_evt & COL_JOY_FIRE1)) {
    return 1;
  }
  delay(5);

  *p_game_sel = game_sel;
  return 0;
}


void
main(void)
{
  uchar  game_sel  = 0;
  uchar  banner_dx = 0;
  uchar  page_num  = 0;

  init_screen();
  init_sound();

  do {
   
    if (game_sel >= GAME_MAX_PAGE) page_num = 2;
    else                           page_num = 1;

    display_banner_dx( banner_dx, page_num );
    banner_dx++;

    display_game_list( game_sel );
    if (get_game_select( &game_sel )) break;

  } while (1);

  load_game( game_sel );
}
